; ---------------------------------------------------------
;	serial communication interface driver
; =========================================================
;	(C)2004 t.hara
; ---------------------------------------------------------

	.cpu	300HA

			.include	"trapper.inc"		; interrupt trapper

; ---------------------------------------------------------
;	Exports
			.export		InitializeSci		; initialize SCI block
			.export		TerminateSci		; terminate SCI block
			.export		SciPutc				; send one character
			.export		SciGetc				; receive one character
			.export		SciPuts				; send ZERO terminated string

; ---------------------------------------------------------
;	Configure
;BAUDRATE	.equ		51					;  9600bps @ 16MHz
;BAUDRATE	.equ		25					; 19200bps @ 16MHz
BAUDRATE	.equ		12					; 38400bps @ 16MHz

FIFOTOP		.equ		H'FFF010			; 256bytes
WR_PTR		.equ		(FIFOTOP+256+0)
RD_PTR		.equ		(FIFOTOP+256+1)

; ---------------------------------------------------------
;	Register define
SMR0		.equ		H'FFFFB0
BRR0		.equ	 	H'FFFFB1
SCR0		.equ	 	H'FFFFB2
TDR0	 	.equ		H'FFFFB3
SSR0	 	.equ		H'FFFFB4
RDR0		.equ		H'FFFFB5
TE0			.bequ		5,SCR0
RE0			.bequ		6,SCR0
TDRE0		.bequ		7,SSR0
RDRF0		.bequ		6,SSR0

SMR1		.equ		H'FFFFB8
BRR1		.equ	 	H'FFFFB9
SCR1		.equ	 	H'FFFFBA
TDR1	 	.equ		H'FFFFBB
SSR1	 	.equ		H'FFFFBC
RDR1		.equ		H'FFFFBD
TE1			.bequ		5,SCR1
RE1			.bequ		6,SCR1
TDRE1		.bequ		7,SSR1
RDRF1		.bequ		6,SSR1

; ---------------------------------------------------------
			.section	rom, code

; ---------------------------------------------------------
;	initialize SCI block
;
InitializeSci:
			push.l		ER0
			orc.b		#B'10000000, CCR	; 

			mov.l		#SciReceiveInt, ER0	; Mgbv
			mov.l		ER0, @LinkRXI1
			mov.l		#SciErrorInt, ER0	; G[gbv
			mov.l		ER0, @LinkERI1

			xor.b		R0L, R0L			; clear FIFO
			mov.b		R0L, @WR_PTR
			mov.b		R0L, @RD_PTR

			mov.b		#H'40, R0L
			mov.b		R0L, @SCR1			; Mgp
			mov.b		#H'00, R0L
			mov.b		R0L, @SMR1			; , 8bit, p, stop1bit, MP֎~, clk
			mov.b		#BAUDRATE, R0L
			mov.b		R0L, @BRR1
			mov.b		#H'70, R0L
			mov.b		R0L, @SCR1			; TE, RE = 1
			andc.b		#B'01111111, CCR	; 
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	terminate SCI block
;
TerminateSci:
			push.w		R0
			orc.b		#B'10000000, CCR	; 

			mov.b		#H'30, R0L			; gp
			mov.b		R0L, @SCR1			; TE, RE = 1
			mov.b		#H'84, R0L
			mov.b		R0L, @SSR1			; Ă邩Ȃv낷

			andc.b		#B'01111111, CCR	; 
			pop.w		R0
			rts

; ---------------------------------------------------------
;	send one character
;
SciPutc:
			push.w		R1
SciPutcWait:
			mov.b		@SSR1, R1L			; check TDRE
			bpl			SciPutcWait			; if TDRE=0 then go to SciPutc
			mov.b		R0L, @TDR1			; send data
			bclr		#7,R1L				; clear TDRE
			mov.b		R1L, @SSR1
			pop.w		R1
			rts

; ---------------------------------------------------------
;	receive one character
;
SciGetc:
			push.l		ER1
			mov.b		@RD_PTR, R1L
SciGetcWait:
			mov.b		@WR_PTR, R1H
			cmp.b		R1L, R1H
			beq			SciGetcWait:8		; FIFOȂ҂
			xor.w		E1, E1
			xor.b		R1H, R1H
			add.l		#FIFOTOP, ER1
			mov.b		@ER1, R0L
			add.b		#((1 - FIFOTOP) & 255), R1L
			mov.b		R1L, @RD_PTR
			andc.b		#B'11111110, CCR	; C = 0
			pop.l		ER1
			rts

; ---------------------------------------------------------
;	send ZERO terminated string
;
SciPuts:
			push.w		R0
			push.l		ER1
SciPutsLoop:
			mov.b		@ER1, R0L
			beq			SciPutsLoopExit
			jsr			@SciPutc
			add.l		#1, ER1
			bra			SciPutsLoop
SciPutsLoopExit:
			pop.l		ER1
			pop.w		R0
			rts

; ---------------------------------------------------------
;	receive one line
;	ER1 ... in  i[obt@̃AhX
;	R0H ... in  ER1 ̎ꏊ̃TCY (1ȏ)
;	ER1 ... out ̎̃AhX ([̃AhX)
;	R0L ... out Ōɓ͂
;	R0H ... out obt@̂ݎgpTCY
;
SciGets:
			dec.b		R0H				; [̕\
			beq			SciGetsLoopExit
SciGetsLoop:
			jsr			@SciGetc
			cmp.b		#32, R0L
			bcs			SciGetsLoopExit
			mov.b		R0L, @ER1
			inc.l		#1, ER1
			dec.b		R0H
			bne			SciGetsLoop
SciGetsLoopExit:
			push.w		R0
			xor.b		R0L, R0L
			mov.b		R0L, @ER1
			pop.w		R0
			rts

; ---------------------------------------------------------
;	receive interrupt
;	ւŌĂ΂, ER0, ER1 ͌ĂяoŕۑĂ
;
SciReceiveInt:
			mov.b		@SSR1, R0L			; 荞ݗv낷
			and.b		#H'BF, R0L
			mov.b		R0L, @SSR1

			mov.b		@RDR1, R0L			; M擾
			mov.b		@WR_PTR, R1L
			mov.b		@RD_PTR, R1H
			dec.b		R1H					; FIFO`FbN
			xor.b		R1L, R1H
			beq			SciReceiveIntExit	; ꂽ牽Ȃ
			xor.b		R1H, R1H			; ߂
			xor.w		E1, E1
			add.l		#FIFOTOP, ER1
			mov.b		R0L, @ER1
			add.b		#((1 - FIFOTOP) & 255), R1L
			mov.b		R1L, @WR_PTR
SciReceiveIntExit:
			rts

; ---------------------------------------------------------
;	error interrupt
;	ւŌĂ΂, ER0, ER1 ͌ĂяoŕۑĂ
;
SciErrorInt:
			mov.b		@SSR1, R0L			; 荞ݗv낷
			and.b		#H'CF, R0L
			mov.b		R0L, @SSR1
			rts

			.end
